﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Application.CallCenter.Calls;
using Hotline.Application.Tels;
using Hotline.Caching.Interfaces;
using Hotline.CallCenter.BlackLists;
using Hotline.CallCenter.Calls;
using Hotline.CallCenter.Tels;
using Hotline.Settings;
using Hotline.Share.Dtos.CallCenter;
using Hotline.Share.Dtos.TrCallCenter;
using Hotline.Share.Enums.CallCenter;
using Microsoft.AspNetCore.Http;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.CallCenter
{
    public class TianRunCallApplication : DefaultCallApplication
    {
        private readonly ISessionContext _sessionContext;
        private readonly IRepository<TrCallRecord> _trCallRecordRepository;
        private readonly ITrApplication _trApplication;
        private readonly ITelApplication _telApplication;
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;

        public TianRunCallApplication(
            ISessionContext sessionContext,
            IRepository<TrCallRecord> trCallRecordRepository,
            ITrApplication trApplication,
            ITelApplication telApplication,
            ISystemSettingCacheManager systemSettingCacheManager
        )
        {
            _sessionContext = sessionContext;
            _trCallRecordRepository = trCallRecordRepository;
            _trApplication = trApplication;
            _telApplication = telApplication;
            _systemSettingCacheManager = systemSettingCacheManager;
        }

        /// <summary>
        /// 新增黑名单
        /// </summary>
        public override async Task<string> AddBlackListAsync(AddBlacklistDto dto, CancellationToken cancellationToken)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// 删除黑名单
        /// </summary>
        public override async Task RemoveBlackListAsync(string id, CancellationToken cancellationToken)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// 查询黑名单
        /// </summary>
        public override async Task<List<Blacklist>> QueryBlackListsAsync(CancellationToken cancellationToken)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// 签入
        /// </summary>
        public new async Task<TrOnDutyResponseDto> SignInAsync(SignInDto dto, CancellationToken cancellationToken) =>
            await _trApplication.OnSign(_sessionContext.RequiredUserId, dto.TelNo, (ETelModel)dto.TelModelState, cancellationToken);

        /// <summary>
        /// 签出
        /// </summary>
        public new Task SingOutAsync(CancellationToken cancellationToken) =>
            _telApplication.SignOutAsync(_sessionContext.RequiredUserId, cancellationToken);

        public new Task SingOutAsync(string telNo, CancellationToken cancellationToken) =>
            _telApplication.SignOutByTelNoAsync(telNo, cancellationToken);

        /// <summary>
        /// 查询当前用户的分机状态
        /// </summary>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public new Task<TrOnDutyResponseDto> GetTelStateAsync(CancellationToken cancellationToken) =>
            _trApplication.TelState(_sessionContext.RequiredUserId, cancellationToken);


        public new async Task<TrCallRecord?> GetTianrunCallAsync(string callId, CancellationToken cancellationToken)
        {
            if (string.IsNullOrEmpty(callId)) return null;
            var callRecord = await _trCallRecordRepository.GetAsync(
                x => x.OtherAccept == callId && string.IsNullOrEmpty(x.OtherAccept) == false, cancellationToken);
            return callRecord;
        }

        /// <summary>
        /// 关联通话记录与order（添润）
        /// </summary>
        public new async Task RelateTianrunCallWithOrderAsync(string callId, string orderId, CancellationToken cancellationToken)
        {
            var callRecord = await _trCallRecordRepository.GetAsync(
                p => p.OtherAccept == callId && string.IsNullOrEmpty(p.OtherAccept) == false, cancellationToken);
            if (callRecord != null && string.IsNullOrEmpty(callRecord.ExternalId))
            {
                callRecord.ExternalId = orderId;
                callRecord.CallOrderType = Share.Enums.CallCenter.ECallOrderType.Order;
                await _trCallRecordRepository.UpdateAsync(callRecord, cancellationToken);
            }
        }

        /// <summary>
        /// 查询通话记录
        /// </summary>
        public new async Task<List<TrCallRecord>> QueryTianrunCallsAsync(
            string? phone,
            ECallDirection? direction,
            DateTime? callStartTimeStart,
            DateTime? callStartTimeEnd,
            CancellationToken cancellationToken)
        {
            return await _trCallRecordRepository.Queryable()
                .WhereIF(!string.IsNullOrEmpty(phone), d => d.CPN == phone || d.CDPN == phone)
                .WhereIF(direction != null, d => d.CallDirection == direction)
                .WhereIF(callStartTimeStart != null, d => d.BeginIvrTime >= callStartTimeStart)
                .WhereIF(callStartTimeEnd != null, d => d.BeginIvrTime <= callStartTimeEnd)
                .OrderBy(x => x.CreatedTime)
                .ToListAsync(cancellationToken);
        }
    }
}